package apps;

import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Text;

import com.ibm.rhapsody.apps.apps.UI3App;
import com.ibm.rhapsody.apps.utils.IOServices;
import com.ibm.rhapsody.apps.utils.RAExpander;
import com.ibm.rhapsody.apps.utils.Reporter;
import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.IRPTag;


public class MainApp extends UI3App {

	protected void initialize() {
		super.initialize();
		
		// Rename controls to create a dialog according to your needs
		sShell.setText("MBDG: Set Template");
		label1.setText("Template Path:");
		text1.setText("");
		buttonOK.setText("Set");
		buttonCancel.setText("Cancel");
		
		initTextField(selected);
	}
	
	/*
	* This method is called on invoking an app inside Rhapsody.
	* rhapsody - Instance of an active Rhapsody application 
	* selected - Selected element in Rhapsody
	* path     - Input in dialog
	*/
	public void execute(IRPApplication rhapsody, IRPModelElement selected, String path) {
		String expandedPath = RAExpander.expand(rhapsody, path);
		if (!IOServices.doesExist(expandedPath) || !expandedPath.endsWith(".dta")) {
			Reporter.report(String.format("Invalid Template Path: \"%s\"", expandedPath));
		} else {
			IRPTag tag = selected.getTag("templateFile");
			if (tag == null) {
				Reporter.report(String.format("Selected element [%] is not document element.", selected.getName()));	
			} else {
				selected.setTagValue(tag, path);
				Reporter.report(String.format("%s.templateFile = %s", selected.getName(), expandedPath));
			}
		}
	}
	
	protected void initTextField(IRPModelElement selected) {
		IRPTag tag = selected.getTag("templateFile");
		if (tag != null) {
			text1.setText(tag.getValue());
		}
	}
	
	protected void chooseFile(Text text) {
		FileDialog dlg = new FileDialog(sShell);

        // Set the initial filter path according
        // to anything they've selected or typed in
		String initDir = RAExpander.expand(getRhapsody(), "${OMROOT}/ModelBasedDocumentGeneration/templates");
        dlg.setFilterPath(initDir);

        // Change the title bar text
        dlg.setText("Choose template file");
        
        String[] extensions = {"*.dta"};
        dlg.setFilterExtensions(extensions);
        
        // Calling open() will open and run the dialog.
        // It will return the selected directory, or
        // null if user cancels
        String path = dlg.open();
        if (path != null) {
        	// Set the text box to the new selection
			RAExpander.setApplication(rhapsody);
			String collapsedPath = RAExpander.collapse(path);
        	text.setText(collapsedPath);
        }		
	}	
	
	
    /*
     *  Debug app by launching it as "Java Application" is Eclipse.
	 *  Note: Select an element in Rhapsody in order to simulate launching app on a selected element in the browser.
     */		
	public static void main(String[] args) {
		MainApp app = new MainApp();
		app.invokeFromMain();
	}
}
