package com.ibm.rhapsody.sysML;

import java.util.LinkedList;
import java.util.List;

import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPAttribute;
import com.telelogic.rhapsody.core.IRPClassifier;
import com.telelogic.rhapsody.core.IRPCollection;
import com.telelogic.rhapsody.core.IRPFlow;
import com.telelogic.rhapsody.core.IRPGraphElement;
import com.telelogic.rhapsody.core.IRPInstance;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.IRPPackage;
import com.telelogic.rhapsody.core.IRPPort;
import com.telelogic.rhapsody.core.IRPRelation;
import com.telelogic.rhapsody.core.IRPStereotype;
import com.telelogic.rhapsody.core.IRPTag;
import com.telelogic.rhapsody.core.RhapsodyAppServer;

public class RhpUtils {
	public static IRPGraphElement getSelectedGraphicalElement(IRPModelElement modelElement) {
		IRPGraphElement selectedElem = null;
		IRPApplication rhapsody = RhapsodyAppServer.getActiveRhapsodyApplication();
		if(rhapsody != null) {
			IRPCollection elems = rhapsody.getSelectedGraphElements();
			if(elems != null && elems.getCount() == 1) {
				Object elem = elems.getItem(1);
				if(elem instanceof IRPGraphElement) {
					IRPGraphElement node = (IRPGraphElement)elem;
					if(modelElement != null) {
						if(node.getModelObject() != null && node.getModelObject().equals(modelElement)) {
							selectedElem = node;
						}
					}
					else
						selectedElem = node;
				}
			}
		}
		return selectedElem;
	}
	
	public static IRPStereotype getStereotype(IRPModelElement elem, String stereotypeName) {
		IRPStereotype stereotype = null;
		if(elem != null && stereotypeName != null && stereotypeName.isEmpty() == false) {
			IRPCollection stereotypes = elem.getStereotypes();
			if(stereotypes != null && stereotypes.getCount() > 0) {
				for(int i = 1 ; i <= stereotypes.getCount() ; i++) {
					IRPStereotype s = (IRPStereotype) stereotypes.getItem(i);
					if(s != null && s.getName().equals(stereotypeName)) {
						stereotype = s;
						break;
					}
				}
			}
		}
		return stereotype;
	}
	
	public static IRPPackage getPackageOfBlock(IRPClassifier block) {
		IRPPackage pkg = null;
		if(block != null) {
			IRPModelElement owner = block.getOwner();
			while(owner != null) {
				if(owner instanceof IRPPackage) {
					pkg = (IRPPackage)owner;
					break;
				}
				owner = owner.getOwner();
			}
		}
		return pkg;
	}
	
	public static void collectFlowProperties(IRPClassifier block, boolean bIn, int bIsReveresed, List<IRPAttribute> flowProperties) {
		if(block != null && (getStereotype(block, "flowSpecification") != null || getStereotype(block, "interfaceBlock") != null)) {
			IRPCollection attrs = block.getAttributesIncludingBases();
			if(attrs != null && attrs.getCount() > 0) {
				if(flowProperties == null)
					flowProperties = new LinkedList<IRPAttribute>();
				for(int i = 1 ; i <= attrs.getCount() ; i++) {
					IRPAttribute attr = (IRPAttribute) attrs.getItem(i);
					if(getStereotype(attr, "flowProperty") != null) {
						IRPClassifier attrType = attr.getType();
						if(attrType != null  && (getStereotype(attrType, "flowSpecification") != null || getStereotype(attrType, "interfaceBlock") != null)) {
							collectFlowProperties(attrType, bIn, bIsReveresed, flowProperties);
						}
						else {
							IRPTag tag = attr.getTag("direction");
							if(tag != null) {
								String direction = tag.getValue();
								if(direction != null) {
									if(bIn) {
										if((direction.equals("In") || direction.equals("Bidirectional") ||
												(bIsReveresed == 1 && direction.equals("Out"))) && !flowProperties.contains(attr)) {
											flowProperties.add(attr);
										}
									}
									else {
										if((direction.equals("Out") || direction.equals("Bidirectional") ||
												(bIsReveresed == 1 && direction.equals("In"))) && !flowProperties.contains(attr)) {
											flowProperties.add(attr);
										}
									}
								}
							}
						}
					}
				}
			}

			if(getStereotype(block, "interfaceBlock") != null) {
				List<IRPPort> proxyPorts = getProxyportsOfBlock(block);
				if(proxyPorts != null && proxyPorts.size() > 0) {
					for(IRPPort proxyPort :proxyPorts) {
						IRPClassifier portType = proxyPort.getOtherClass();
						if(portType != null)
							collectFlowProperties(portType, bIn, bIsReveresed, flowProperties);
					}
				}
			}
		}
	}
	
	public static List<IRPInstance> getAllPartsOfBlock(IRPClassifier block) {
		List<IRPInstance> parts = null;
		IRPCollection relations = block.getRelations();
		if(relations != null && relations.getCount() > 0) {
			for(int i = 1 ; i <= relations.getCount() ; i++) {
				IRPRelation rel = (IRPRelation)relations.getItem(i);
				if(rel instanceof IRPInstance) {
					IRPInstance part = (IRPInstance)rel;
					
					if(parts == null)
						parts = new LinkedList<IRPInstance>();
					
					parts.add(part);		
				}
			}
		}
		return parts;
	}
	
	public static IRPFlow findFlow(IRPPackage pkg, String flowName) {
		IRPFlow flow = null;
		if(pkg != null) {
			IRPCollection flows = pkg.getFlows();
			if(flows != null && flows.getCount() > 0) {
				for(int i = 1 ; i <= flows.getCount() ; i++) {
					IRPFlow tmpFlow = (IRPFlow)flows.getItem(i);
					if(tmpFlow != null && tmpFlow.getName().equals(flowName)) {
						flow = tmpFlow;
						break;
					}
				}
			}
		}
		return flow;
	}
	
	public static List<IRPPort> getProxyportsOfBlock(IRPClassifier block) {
		List<IRPPort> proxyports = null;
		if(block != null) {
			IRPCollection ports = block.getPorts();
			if(ports != null && ports.getCount() > 0) {
				for(int j = 1 ; j <= ports.getCount() ; j++) {
					IRPPort port = (IRPPort) ports.getItem(j);
					IRPCollection stereotypes = port.getStereotypes();
					if(stereotypes != null && stereotypes.getCount() > 0) {
						for(int i = 1 ; i <= stereotypes.getCount() ; i++) {
							IRPStereotype s = (IRPStereotype) stereotypes.getItem(i);
							if(s != null && s.getName().equals("proxyPort")) {
								if(proxyports == null)
									proxyports = new LinkedList<IRPPort>();
								proxyports.add(port);
								break;
							}
						}
					}
				}
			}
		}
		return proxyports;
	}
}
