package com.ibm.rhapsody.sysML.proxyportWizards;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.StringTokenizer;
import java.util.Vector;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.TreeEditor;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.swt.widgets.TreeItem;

import com.ibm.rhapsody.sysML.proxyportWizards.Utils.DirectionType;
import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPAssociationClass;
import com.telelogic.rhapsody.core.IRPAttribute;
import com.telelogic.rhapsody.core.IRPClassifier;
import com.telelogic.rhapsody.core.IRPCollection;
import com.telelogic.rhapsody.core.IRPDiagram;
import com.telelogic.rhapsody.core.IRPFlow;
import com.telelogic.rhapsody.core.IRPGraphEdge;
import com.telelogic.rhapsody.core.IRPGraphNode;
import com.telelogic.rhapsody.core.IRPInstance;
import com.telelogic.rhapsody.core.IRPLink;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.IRPPackage;
import com.telelogic.rhapsody.core.IRPPort;
import com.telelogic.rhapsody.core.IRPRelation;
import com.telelogic.rhapsody.core.IRPTableLayout;
import com.telelogic.rhapsody.core.IRPTableView;
import com.telelogic.rhapsody.core.IRPTag;
import com.telelogic.rhapsody.core.RhapsodyRuntimeException;
import org.eclipse.swt.widgets.Button;

public class MappingProxyportsPage extends WizardPage {

	private CreateProxyportPage page1 = null;
	private CreateProxyportPage page2 = null;
	private String end1InterfaceName = null;
	private String end2InterfaceName = null;
	
	private Composite container = null;
	private Tree tree;
	private TreeColumn end1TreeColumn = null;
	private TreeColumn end2TreeColumn = null;
	private Button btnCheckButton = null;
	private Vector<TreeEditor> treeEditors = new Vector<TreeEditor>();
	
	private IRPApplication rhapsody = null;
	private IRPLink connector = null;
	private IRPPort fromPort = null;
	private IRPPort toPort = null;
	private IRPClassifier fromPortType = null;
	private IRPClassifier toPortType = null;
	private IRPInstance fromPart = null;
	private IRPInstance toPart = null;
	private IRPAssociationClass mappingAssocClass = null;
	
	public MappingProxyportsPage(CreateProxyportPage page1, CreateProxyportPage page2, IRPLink connector) {
		super("MappingProxyportsPage");
		
		this.page1 = page1;
		this.page2 = page2;
		this.connector = connector;
		
		setTitle("Property mapping");
		
		setPageDescription();
		setPageComplete(false);
	}
	
	private void setPageDescription() {
		String description = "Map the properties of '" + page1.getInterfaceName() + "' to the properties of '" + page2.getInterfaceName() + "'";
		setDescription(description);
	}
	
	private void updateColumnsTitleAndSize() {
		if(end1TreeColumn != null && end2TreeColumn != null) {
			String end1Title = end1InterfaceName + " Properties";
			if(page1 != null && page1.getPortName() != null)
				end1Title += " (Proxy Port " + page1.getPortName() + ")";
			end1TreeColumn.setText(end1Title);
			GC gc = new GC(tree);
			int end1TitleWidth = gc.stringExtent(end1Title).x;
			end1TreeColumn.setWidth(end1TitleWidth + 15);
		
			String end2Title = end2InterfaceName + " Properties";
			if(page2 != null && page2.getPortName() != null)
				end2Title += " (Proxy Port " + page2.getPortName() + ")";
			end2TreeColumn.setText(end2Title);
			int end2TitleWidth = gc.stringExtent(end2Title).x;
			end2TreeColumn.setWidth(end2TitleWidth + 15);
			
			gc.dispose();
		}
	}

	public void updatePageIfNeeded() {
		if(page1 == null || page2 == null)
			return;
		
		end1InterfaceName = page1.getInterfaceName();
		end2InterfaceName = page2.getInterfaceName();
			
		if(end1InterfaceName != null && end2InterfaceName != null) {
			setPageDescription();
			createContents();
			updateColumnsTitleAndSize();
			setPageComplete(true);
		}
		
		fillTree();
	}
	
	/**
	 * Create contents of the wizard.
	 * @param parent
	 */
	public void createControl(Composite parent) {
		container = new Composite(parent, SWT.NULL);
		setControl(container);
		
		createContents();
		fillTree();
	}

	private void createContents() {
		if(container == null || end1InterfaceName == null || end2InterfaceName == null ||
				page1 == null || page2 == null || tree != null) {
			return;
		}
		
		container.setLayout(new GridLayout(3, false));
		
		tree = new Tree(container, SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION);
		tree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 3, 1));
		tree.setHeaderVisible(true);
		tree.setLinesVisible(true);
		
		end1TreeColumn = new TreeColumn(tree, SWT.NONE);
					
		end2TreeColumn = new TreeColumn(tree, SWT.NONE);
		
		updateColumnsTitleAndSize();
		
		btnCheckButton = new Button(container, SWT.CHECK);
		btnCheckButton.setText("Create Mapping Graphics");
				
		container.layout();
	}
	
	private void fillTree() {
		if(container == null || end1InterfaceName == null || end2InterfaceName == null)
			return;
			
		Map<String, IRPAttribute> fromInterfaceAttrsMap = page1.getInterfaceFlowPropertiesAsMap();
		Map<String,DirectionType> fromAttrsDirectionMap = page1.getInterfaceFlowPropertiesDirectionMap();
		if(fromAttrsDirectionMap != null && fromAttrsDirectionMap.size() > 0) {
			emptyTree();
			
			TreeItem attrRootItem = Utils.getOrCreateTreeRootItem(tree, "Attributes");
			
			for (Map.Entry<String,DirectionType> entry : fromAttrsDirectionMap.entrySet()) {
				String fromAttrName = entry.getKey();
				DirectionType fromAttrDirection = entry.getValue();
				
				TreeItem treeItem = Utils.getOrCreateTreeItem(attrRootItem, fromAttrName, null);
				if(fromInterfaceAttrsMap != null) {
					IRPAttribute interfaceAttr = fromInterfaceAttrsMap.get(fromAttrName);
					if(interfaceAttr != null)
						treeItem.setData(interfaceAttr);
				}
			    CCombo combo = null;
			    String oldSelection = null;
			    TreeEditor editor = getTreeEditor(treeItem, 1);
			    if(editor == null) {
			    	editor = new TreeEditor(tree);
					combo = new CCombo(tree, SWT.READ_ONLY);	
					editor.grabHorizontal = true;
				    editor.setEditor(combo, treeItem, 1);
				    treeEditors.add(editor);  
			    }
			    else {
			    	combo = (CCombo)editor.getEditor();
			    	oldSelection = combo.getText();
			    	combo.removeAll();
			    }
			    fillEnd2AttributeCombo(combo, treeItem, oldSelection, fromAttrDirection);
			}	
			attrRootItem.setExpanded(true);
		 }
		
		Map<String, IRPPort> fromInterfacePorts = page1.getInterfacePortsAsMap();
		if(fromInterfacePorts != null && fromInterfacePorts.size() > 0) {
			TreeItem portRootItem = Utils.getOrCreateTreeRootItem(tree, "Ports");
						
			Iterator<Entry<String, IRPPort>> it = fromInterfacePorts.entrySet().iterator();
		    while (it.hasNext()) {
		        Map.Entry<String, IRPPort> entry = (Map.Entry<String, IRPPort>)it.next();
		        String interfacePortName = (String) entry.getKey();
		        IRPPort blockPort = (IRPPort) entry.getValue();
		        
		        TreeItem interfacePortItem = Utils.getOrCreateTreeItem(portRootItem, interfacePortName, null);
		        CCombo combo = null;
		        String oldSelection = null;
				TreeEditor editor = getTreeEditor(interfacePortItem, 1);
				if(editor == null) {
				   	editor = new TreeEditor(tree);
					combo = new CCombo(tree, SWT.READ_ONLY);	
					editor.grabHorizontal = true;
				    editor.setEditor(combo, interfacePortItem, 1);
				    treeEditors.add(editor);  
				}
				else {
				   	combo = (CCombo)editor.getEditor();
				   	oldSelection = combo.getText();
				   	combo.removeAll();
				}
				fillEnd2PortCombo(combo, interfacePortItem, oldSelection);
				
		        it.remove(); // avoids a ConcurrentModificationException
		        
		        populateInnerPortType(interfacePortItem, blockPort.getOtherClass());
		    }
		    portRootItem.setExpanded(true);
		}
	}
	
	private void emptyTree() {
		for(TreeEditor te : treeEditors) {
			Control ctrl = te.getEditor();
			if(ctrl != null && ctrl.isDisposed() == false)
				ctrl.dispose();
			te.setItem(null);
		}
		treeEditors.removeAllElements();
		
		tree.removeAll();
	}
	
	private void fillEnd2AttributeCombo(CCombo combo, TreeItem treeItem, String oldSelection, DirectionType fromAttrDirection) {
		if(combo != null && page2 != null && treeItem != null) {
			DirectionType direction = DirectionType.InOut;
			if(fromAttrDirection == DirectionType.Out)
				direction = DirectionType.In;
			else if(fromAttrDirection == DirectionType.In)
				direction = DirectionType.Out;
			
			Vector<String> toAttrs = page2.getInterfaceFlowPropertiesAsString(direction);
			if(toAttrs != null && toAttrs.size() > 0) {
				combo.add("");
				for(int j = 0  ; j < toAttrs.size() ; j++) {
					String toAttrName = toAttrs.get(j);
					combo.add(toAttrName);
				}
			}
			
			Map<String, IRPPort> toInterfacePorts = page2.getInterfacePortsAsMap();
			if(toInterfacePorts != null && toInterfacePorts.size() > 0) {
				Iterator<Entry<String, IRPPort>> it = toInterfacePorts.entrySet().iterator();
			    while (it.hasNext()) {
			        Map.Entry<String, IRPPort> entry = (Map.Entry<String, IRPPort>)it.next();
			        String interfacePortName = (String) entry.getKey();
			        IRPPort blockPort = (IRPPort) entry.getValue();
			        fillEnd2InnerPortsAttributes(blockPort.getOtherClass(), combo, interfacePortName, null);
			    }
			}
			
			selectExistingMappingInCombo(combo, treeItem, oldSelection);
		}
	}
	
	private void fillEnd2InnerPortsAttributes(IRPClassifier portType, CCombo combo, String portContextStr, List<IRPModelElement> portContext) {
		if(portType != null && page2 != null && combo != null && portContextStr != null) {
			IRPCollection attrs = portType.getAttributes();
			if(attrs != null && attrs.getCount() > 0) {
				for(int i = 1 ; i <= attrs.getCount() ; i++) {
					if(combo.getItemCount() == 0)
						combo.add("");
					IRPAttribute attr = (IRPAttribute)attrs.getItem(i);
					String attrNameAndType = Utils.getNameAndTypeString(attr);
					String comboItem = attrNameAndType;
					if(portContextStr.isEmpty() == false)
						comboItem = portContextStr + "." + comboItem;
					combo.add(comboItem);
					
					List<IRPModelElement> context = new ArrayList<IRPModelElement>();
					if(portContext != null) {
						for(int j = 0 ; j < portContext.size() ; j++) {
							context.add(portContext.get(j));
						}
					}
					context.add(attr);
					combo.setData(comboItem, context);
				}
			}
			
			List<IRPPort> ports = Utils.getProxyportsOfBlock(portType);
			if(ports != null && ports.size() > 0) {
				for(IRPPort port : ports) {
					String portName = port.getName();
					String newPortContext = portName;
					if(portContextStr.isEmpty() == false)
						newPortContext = portContextStr + "." + portName;
					if(portContext == null)
						portContext = new ArrayList<IRPModelElement>();
					portContext.add(port);
					
					fillEnd2InnerPortsAttributes(port.getOtherClass(), combo, newPortContext, portContext);
				}
			}
		}
	}

	private void fillEnd2PortCombo(CCombo combo, TreeItem treeItem, String oldSelection) {
		if(combo != null && page2 != null) {						
			Map<String, IRPPort> toInterfacePorts = page2.getInterfacePortsAsMap();
			if(toInterfacePorts != null && toInterfacePorts.size() > 0) {
				if(combo.getItemCount() == 0)
					combo.add("");
				Iterator<Entry<String, IRPPort>> it = toInterfacePorts.entrySet().iterator();
			    while (it.hasNext()) {
			        Map.Entry<String, IRPPort> entry = (Map.Entry<String, IRPPort>)it.next();
			        String interfacePortName = (String) entry.getKey();
			        IRPPort blockPort = (IRPPort) entry.getValue();
			        combo.add(interfacePortName);
			        fillEnd2InnerPorts(blockPort.getOtherClass(), combo, interfacePortName, null);
			    }
			}
			selectExistingMappingInCombo(combo, treeItem, oldSelection);
	   }
	}
	
	private void fillEnd2InnerPorts(IRPClassifier portType, CCombo combo, String portContextStr, List<IRPModelElement> portContext) {
		if(portType != null && page2 != null && combo != null && portContextStr != null) {
			List<IRPPort> ports = Utils.getProxyportsOfBlock(portType);
			if(ports != null && ports.size() > 0) {
				for(IRPPort port : ports) {
					String portName = port.getName();
					String newPortContext = portName;
					if(portContextStr.isEmpty() == false)
						newPortContext = portContextStr + "." + portName;
					combo.add(newPortContext);
					
					List<IRPModelElement> context = new ArrayList<IRPModelElement>();
					if(portContext != null) {
						for(int j = 0 ; j < portContext.size() ; j++) {
							context.add(portContext.get(j));
						}
					}
					context.add(port);
					combo.setData(newPortContext, context);
					
					
					if(portContext == null)
						portContext = new ArrayList<IRPModelElement>();
					portContext.add(port);
					
					fillEnd2InnerPorts(port.getOtherClass(), combo, newPortContext, portContext);
				}
			}
		}
	}
	
	private void populateInnerPortType(TreeItem interfacePortItem, IRPClassifier portType) {
		if(interfacePortItem != null && portType != null) {
			IRPCollection attrs = portType.getAttributes();
			if(attrs != null && attrs.getCount() > 0) {
				TreeItem attrRootItem = Utils.getOrCreateTreeItem(interfacePortItem, "Attributes", null);
				
				for(int i = 1 ; i <= attrs.getCount() ; i++) {
					IRPAttribute attr = (IRPAttribute)attrs.getItem(i);
					String attrNameAndType = Utils.getNameAndTypeString(attr);
					TreeItem attrItem = Utils.getOrCreateTreeItem(attrRootItem, attrNameAndType, null);
					attrItem.setData(attr);
					
					CCombo combo = null;
					String oldSelection = null;
					TreeEditor editor = getTreeEditor(attrItem, 1);
					if(editor == null) {
					   	editor = new TreeEditor(tree);
						combo = new CCombo(tree, SWT.READ_ONLY);	
						editor.grabHorizontal = true;
					    editor.setEditor(combo, attrItem, 1);
					    treeEditors.add(editor);  
					}
					else {
					   	combo = (CCombo)editor.getEditor();
					   	oldSelection = combo.getText();
					   	combo.removeAll();
					}
					DirectionType direction = Utils.getDirectionType(attr);
					fillEnd2AttributeCombo(combo, attrItem, oldSelection, direction);
				}
				attrRootItem.setExpanded(true);
			}
			
			List<IRPPort> ports = Utils.getProxyportsOfBlock(portType);
			if(ports != null && ports.size() > 0) {
				TreeItem portRootItem = Utils.getOrCreateTreeItem(interfacePortItem, "Ports", null);
				
				for(IRPPort port : ports) {
					String portName = port.getName();
					TreeItem portItem = Utils.getOrCreateTreeItem(portRootItem, portName, null);
					portItem.setData(port);
					
					CCombo combo = null;
					String oldSelection = null;
					TreeEditor editor = getTreeEditor(portItem, 1);
					if(editor == null) {
					   	editor = new TreeEditor(tree);
						combo = new CCombo(tree, SWT.READ_ONLY);	
						editor.grabHorizontal = true;
					    editor.setEditor(combo, portItem, 1);
					    treeEditors.add(editor);  
					}
					else {
					   	combo = (CCombo)editor.getEditor();
					   	oldSelection = combo.getText();
					   	combo.removeAll();
					}
					fillEnd2PortCombo(combo, portItem, oldSelection);
					
					//recursive call for inner ports
					populateInnerPortType(portItem, port.getOtherClass());
				}
				portRootItem.setExpanded(true);
			}
			interfacePortItem.setExpanded(true);
		}
	}

	private void selectExistingMappingInCombo(CCombo combo, TreeItem treeItem, String oldSelection) {
		//check if a mapping already exist, if so select the mapping
		String fullEnd2Name = null;
		IRPAssociationClass existingAssocClass = getExistingAssociationClass();
	    if(existingAssocClass != null && treeItem != null) {
	       	List<IRPModelElement> end1Context = getContextFromTree(treeItem.getParentItem(), null, page1.getInterfaceBlock());
	       	String itemName = treeItem.getText();
	       	IRPModelElement end1 = null;
	       	if(treeItem.getData() != null && treeItem.getData() instanceof IRPModelElement)
	       		end1 = (IRPModelElement)treeItem.getData();
	       	
	       	IRPFlow flow = Utils.findFlowByEnd1NameAndContext(existingAssocClass, itemName, end1Context, end1);
	       	if(flow != null) {
	       		fullEnd2Name = Utils.getFullEnd2Name(flow, end1Context);	
	       	}   	
	 	}
	    
	    if(fullEnd2Name != null || oldSelection != null) {
   			for(int i = 0 ; i < combo.getItemCount() ; i++) {
   				String comboItem = combo.getItem(i);
   				if(comboItem != null && (comboItem.equals(fullEnd2Name) || comboItem.equals(oldSelection))) {
   					combo.select(i);
   					if(comboItem.equals(fullEnd2Name))
   						combo.setEnabled(false);
   					break;
   				}
   			}
   		}
	}
	
	public void createMapping(IRPApplication rhapsody, IRPLink connector, IRPPort fromPort, IRPPort toPort) {
		if(rhapsody == null || connector == null || fromPort == null || 
				toPort == null || Utils.isProxyport(fromPort) == false || Utils.isProxyport(toPort) == false) {
			return;
		}
		
		this.rhapsody = rhapsody;
		this.connector = connector;
		this.fromPort = fromPort;
		this.toPort = toPort;
		this.fromPart = connector.getFrom();
		this.toPart = connector.getTo();
		
		fromPortType = fromPort.getContract();
		toPortType = toPort.getContract();
		if(fromPortType == null || toPortType == null || 
				Utils.isInterfaceBlock(fromPortType) == false || Utils.isInterfaceBlock(toPortType) == false) {
			return;
		}
		
		mappingAssocClass = null;
		for(TreeEditor te : treeEditors) {
			TreeItem treeItem = te.getItem();
			if(treeItem != null && te.getEditor() instanceof CCombo) {
				String treeItemText = treeItem.getText();
				CCombo combo = (CCombo)te.getEditor();
				if(isTypeTreeItem(treeItem, "Attributes")) {
					IRPAttribute fromAttr = null;
					List<IRPModelElement> fromAttrContext = getContextFromTree(treeItem.getParentItem(), null, fromPortType);
					if(treeItem.getData() != null && treeItem.getData() instanceof IRPAttribute) {
						fromAttr = (IRPAttribute)treeItem.getData();
					}
					else {
						fromAttr = Utils.findAttributeByNameAndType(fromPortType, treeItemText);
					}
						
					String toAttrNameAndType = combo.getText();
					IRPAttribute toAttr = null;
					List<IRPModelElement> toAttrContext = getContextFromComboItem(combo, toAttrNameAndType);
					if(combo.getData(toAttrNameAndType) instanceof List<?>) {
						List<?> contextList = (List<?>)combo.getData(toAttrNameAndType);
						if(contextList.size() > 0) {
							Object elem = contextList.get(contextList.size()-1);
							if(elem instanceof IRPAttribute)
								toAttr = (IRPAttribute)elem;
						}
					}
					else
						toAttr = Utils.findAttributeByNameAndType(toPortType, toAttrNameAndType);
					
					if(fromAttr != null && toAttr != null) {
						if(mappingAssocClass == null) {
							mappingAssocClass = createAssociationClass();											
						}
						
						createMappingBindingConnector(fromAttr, fromAttrContext, toAttr, toAttrContext);
					}
				}
				else if(isTypeTreeItem(treeItem, "Ports")) {
					IRPPort end1Port = null;
					List<IRPModelElement> fromPortContext = getContextFromTree(treeItem.getParentItem(), null, fromPortType);
					if(treeItem.getData() != null && treeItem.getData() instanceof IRPPort) {
						end1Port = (IRPPort)treeItem.getData();
					}
					else {
						end1Port = Utils.findProxyportByName(fromPortType, treeItemText);
					}
						
					IRPPort end2Port = null;
					List<IRPModelElement> toPortContext = null;
					String toPortName = combo.getText();
					if(toPortName != null && toPortName.isEmpty() == false) {
						toPortContext = getContextFromComboItem(combo, toPortName);
						if(combo.getData(toPortName) instanceof List<?>) {
							List<?> contextList = (List<?>)combo.getData(toPortName);
							if(contextList.size() > 0) {
								Object elem = contextList.get(contextList.size()-1);
								if(elem instanceof IRPPort)
									end2Port = (IRPPort)elem;
							}
						}
						else {
							end2Port = Utils.findProxyportByName(toPortType, toPortName);
						}
					}
					
					if(end1Port != null && end2Port != null) {
						if(mappingAssocClass == null) {
							mappingAssocClass = createAssociationClass();											
						}
							
						createMappingBindingConnector(end1Port, fromPortContext, end2Port, toPortContext);
					}
				}
			}
		}
		
		createNewConnectorAccordingToPorts();
		createMappingTable();
		createMappingGraphics();
		rhapsody.refreshAllViews();
	}

	private List<IRPModelElement> getContextFromTree(TreeItem treeItem, List<IRPModelElement> context, IRPClassifier fromPortType) {
		if(treeItem == null)
			return context;
		
		context = getContextFromTree(treeItem.getParentItem(), context, fromPortType);
		if(treeItem.getData() != null && treeItem.getData() instanceof IRPPort) {
			if(context == null)
				context = new ArrayList<IRPModelElement>();
			IRPModelElement elem = (IRPModelElement)treeItem.getData();
			context.add(elem);
		}
		else if(treeItem.getText() != null && isTypeTreeItem(treeItem, "Ports") && fromPortType != null) {
			String treeItemText = treeItem.getText();
			IRPPort port = Utils.findProxyportByName(fromPortType, treeItemText);
			if(port != null) {
				if(context == null)
					context = new ArrayList<IRPModelElement>();
				context.add(port);
			}
		}
		return context;
	}
	
	private boolean isTypeTreeItem(TreeItem treeItem, String type) {
		boolean bIsPortTreeItem = false;
		if(treeItem != null && type != null) {
			TreeItem parentTreeItem = treeItem.getParentItem();
			if(parentTreeItem != null) {
				String parentText = parentTreeItem.getText();
				if(parentText != null && parentText.equals(type)) {
					bIsPortTreeItem = true;
				}
			}
		}
		return bIsPortTreeItem;
	}
	
	private List<IRPModelElement> getContextFromComboItem(CCombo combo, String comboItem) {
		List<IRPModelElement> context = null;
		if(combo != null && comboItem != null && comboItem.isEmpty() == false) {
			Object data = combo.getData(comboItem);
			if(data != null && data instanceof List<?>) {
				List<?> list = (List<?>)data;
				if(list.size() > 0) {
					context = new ArrayList<IRPModelElement>();
					
					StringTokenizer st = new StringTokenizer(comboItem, ".");
					if(st != null && st.hasMoreTokens()) {
						String firstContext = st.nextToken();
						IRPPort port = Utils.findProxyportByName(toPortType, firstContext);
						if(port != null)
							context.add(port);
					}
					
					for(int i = 0 ; i < list.size() - 1 ; i++) {
						Object obj = list.get(i);
						if(obj instanceof IRPModelElement) {
							context.add((IRPModelElement)obj);
						}
					}
				}
			}
		}
		return context;
	}
	
	private void createMappingBindingConnector(IRPModelElement end1, List<IRPModelElement> end1Context,
			IRPModelElement end2, List<IRPModelElement> end2Context) {
		if(end1 != null && end2 != null && mappingAssocClass != null && fromPort != null && toPort != null) {
			String flowName = Utils.createFlowNameWithContext(end1, end1Context, end2, end2Context);
			IRPFlow flow = Utils.findFlow(mappingAssocClass, end1, end2);
			IRPPackage pkg = Utils.getPackageOfBlock(mappingAssocClass);
			if(flow == null && pkg != null)
				flow = Utils.findFlow(pkg, flowName);
			if(flow != null) {
				if(flow.getName().equals(flowName) == false)
					flow.setName(flowName);
				return;
			}
			
			try {
				flow = mappingAssocClass.addFlows(flowName);
			}
			catch(RhapsodyRuntimeException e) {
				if(pkg != null) 
					flow = pkg.addFlows(flowName);
			}
			if(flow != null) {
				flow.setEnd1(end1);
				flow.setEnd2(end2);
				flow.changeTo("BindingConnector");
				IRPTag tag = flow.getTag("SourceContext");
				IRPInstance participantProperty1 = Utils.getParticipantPropertyOf(mappingAssocClass, mappingAssocClass.getEnd2());
				if(tag != null && participantProperty1 != null) {
					tag = flow.setTagElementValue(tag, participantProperty1);
					Utils.addContextToTag(tag, end1Context);
				}
				
				tag = flow.getTag("TargetContext");
				IRPInstance participantProperty2 = Utils.getParticipantPropertyOf(mappingAssocClass, mappingAssocClass.getEnd1());
				if(tag != null && participantProperty2 != null) {
					tag = flow.setTagElementValue(tag, participantProperty2);
					Utils.addContextToTag(tag, end2Context);
				}
			}
		}
	}
	
	private void createNewConnectorAccordingToPorts() {
		if(rhapsody == null || connector == null || fromPort == null || toPort == null ||
				 fromPart == null || toPart == null || mappingAssocClass == null)
			return;
		
		IRPGraphNode fromPartNode = null;
		IRPGraphNode toPartNode = null;
		IRPGraphNode fromPortNode = null;
		IRPGraphNode toPortNode = null;
		//now create the graphical representation of the proxyport and the new connector
		IRPDiagram diagram = rhapsody.getDiagramOfSelectedElement();
		IRPGraphEdge edge = (IRPGraphEdge) Utils.getSelectedGraphicalElement(connector);
		if(edge != null && diagram == null)
			diagram = edge.getDiagram();
		if(edge != null && diagram != null) {
			Point ptSrc = Utils.getSourceTargetPoint(edge, false);
			Point ptTrg = Utils.getSourceTargetPoint(edge, true);
			
			fromPartNode = (IRPGraphNode)edge.getSource();
			if(fromPartNode != null) {
				fromPortNode = Utils.getGraphicalNode(diagram, fromPort, fromPartNode, fromPart);
				if(fromPortNode != null)
					Utils.moveNodeToPoint(fromPortNode, ptSrc);
				else
					fromPortNode = diagram.addNewNodeForElement(fromPort, ptSrc.x - 6, ptSrc.y - 6, 12, 12);
			}
			toPartNode = (IRPGraphNode)edge.getTarget();
			if(toPartNode != null) {
				toPortNode = Utils.getGraphicalNode(diagram, toPort, toPartNode, toPart);
				if(toPortNode != null)
					Utils.moveNodeToPoint(toPortNode, ptTrg);
				else
					toPortNode = diagram.addNewNodeForElement(toPort, ptTrg.x - 6, ptTrg.y - 6, 12, 12);
			}
		}
		
		IRPLink newConnector = createNewConnectorIfNeeded();
		if(newConnector != null) {		
			if(fromPortNode != null && toPortNode != null && diagram != null) {
				Point ptNewSrc = Utils.getMiddlePointOfGraphicalNode(fromPortNode, true);
				Point ptNewTrg = Utils.getMiddlePointOfGraphicalNode(toPortNode, true);
				if(ptNewSrc != null && ptNewTrg != null) {
					diagram.addNewEdgeForElement(newConnector, fromPortNode, ptNewSrc.x, ptNewSrc.y, 
						toPortNode, ptNewTrg.x, ptNewTrg.y);	
				}
			}
			newConnector.highLightElement();
		}	
	}
	
	private IRPLink createNewConnectorIfNeeded() {
		IRPLink newConnector = null;
		if(connector != null && fromPort != null && toPort != null && fromPart != null && toPart != null && mappingAssocClass != null) {
			IRPPort oldFromPort = connector.getFromPort();
			IRPPort oldToPort = connector.getToPort();
			IRPInstance oldFromPart = connector.getFrom();
			IRPInstance oldToPart = connector.getTo();
			IRPRelation newInstantiates = mappingAssocClass.getEnd1();
			if(oldFromPort != null && oldFromPort.equals(fromPort) && oldToPort != null && oldToPort.equals(toPort) && 
					oldFromPart != null && oldFromPart.equals(fromPart) && oldToPart != null && oldToPart.equals(toPart)) {		
				IRPRelation currentInstantiates = connector.getInstantiates();
				if(currentInstantiates == null || currentInstantiates.equals(newInstantiates) == false)
					connector.setInstantiates(newInstantiates);
			}
			else {
				connector.deleteFromProject();
				newConnector = fromPart.addLinkToElement(toPart, newInstantiates, fromPort, toPort);
				newConnector.changeTo("connector");	
				connector = newConnector;
			}
		}
		return newConnector;
	}
	
	private IRPAssociationClass createAssociationClass() {
		IRPAssociationClass assocClass = null;
		IRPPackage pkg = Utils.getPackageOfBlock(fromPortType);
		if(rhapsody == null || fromPort == null || fromPortType == null || 
				toPort == null || toPortType == null || pkg == null)
			return null;
		
		//first check is an associationClass already exist
		assocClass = getExistingAssociationClass();
		
		if(assocClass == null) {
			String assocClassName = fromPortType.getName() + "_" +  toPortType.getName();
			String roleName1 = "its" + toPortType.getName();
			String roleName2 = "its" + fromPortType.getName();
			String linkName = fromPortType.getName() + "_" + toPortType.getName();
			IRPRelation end1 = fromPortType.addRelationTo(toPortType, roleName1, "Association", "1", roleName2, "Association", "1", linkName);
			if(end1 != null) {
				IRPRelation end2 = end1.getInverse();
				if(end2 != null) {
					assocClass = pkg.addAssociation(end1, end2, assocClassName);
				//	assocClass.setPropertyValue("ObjectModelGe.Class.ShowPorts", "False");
				//	assocClass.setPropertyValue("ObjectModelGe.Interface.ShowPorts", "False");
				//	assocClass.setPropertyValue("ObjectModelGe.Object.ShowPorts", "False");
				//	assocClass.setIsClass(1);
				}
			}
		}
		return assocClass;
	}
	
	private IRPAssociationClass getExistingAssociationClass() {
		IRPAssociationClass existingAssocClass = null;
		if(connector != null && page1 != null) {
			IRPClassifier IB1 = page1.getInterfaceBlock();
			IRPRelation end1 = connector.getInstantiates();
			if(end1 != null && IB1 != null) {
				IRPPackage pkg = Utils.getPackageOfBlock(IB1);
				IRPRelation end2 = end1.getInverse();
				if(end2 != null && pkg != null) {
					IRPCollection assocClasses = pkg.getAssociationClasses();
					if(assocClasses != null && assocClasses.getCount() > 0) {
						for(int i = 1 ; i <= assocClasses.getCount() ; i++) {
							IRPAssociationClass tmpAssocClass = (IRPAssociationClass) assocClasses.getItem(i);
							if(end1.equals(tmpAssocClass.getEnd1()) && end2.equals(tmpAssocClass.getEnd2())) {
								existingAssocClass = tmpAssocClass;
								break;
							}
						}
					}
				}
			}
		}
		return existingAssocClass;
	}
	
	private TreeEditor getTreeEditor(TreeItem item, int column) {
		TreeEditor editor = null;
		if(item != null) {
			for(TreeEditor te : treeEditors) {
				if(te.getItem().equals(item) && te.getColumn() == column) {
					editor = te;
					break;
				}
			}
		}
		return editor;
	}	
	
	private void createMappingTable() {
		if(mappingAssocClass != null) {
			IRPPackage pkg = Utils.getPackageOfBlock(mappingAssocClass);
			IRPTableLayout ioTableLayout = Utils.getInputOutputTableLayout();
			if(pkg != null && ioTableLayout != null) {
				String tableViewName = mappingAssocClass.getName() + "_InputOutputTable";
				IRPTableView tableView = Utils.getOrCreateTableView(pkg, tableViewName, ioTableLayout, mappingAssocClass);
				if(tableView != null) {
						
					//now create a HyperLink between the mappingAssocClass and the tableView
					Utils.getOrCreateHyperLink(mappingAssocClass, tableView);
					
					//now create a HyperLink between the connector and the tableView	
					if(connector != null) {
						Utils.getOrCreateHyperLink(connector, tableView);
					}
				}
			}
		}
	}
	
	private void createMappingGraphics() {
		if(btnCheckButton == null || btnCheckButton.getSelection() == false || mappingAssocClass == null)
			return;
		
		MappingGraphicsCreator graphicsCreator = new MappingGraphicsCreator(mappingAssocClass, rhapsody);
		graphicsCreator.createGraphics();
	}
}
